/*
 * \brief  Obtain formatted timestamps
 * \author Timo Nicolai
 * \date   2022-11-07
 */

/*
 * Copyright (C) 2022 gapfruit AG
 */

#pragma once

/* Genode includes */
#include <base/log.h>
#include <libc/component.h>
#include <util/string.h>

/* libc includes */
#include <stddef.h>
#include <string.h>
#include <time.h>


namespace Azure_iot_client {

	enum : Genode::size_t { MAX_TIMESTAMP_LEN = 32 };

	using Timestamp = Genode::String<MAX_TIMESTAMP_LEN + 1>;

	/* Timestamp format used by Azure device twin. We add zeros for subsecond
	 * decimals for compatibility. */
	static constexpr char TIMESTAMP_FMT_UTC_AZURE[] = "%Y-%m-%dT%H:%M:%S.0000000Z";
	/* Compact timestamp format intended to be used in filenames etc. */
	static constexpr char TIMESTAMP_FMT_UTC_COMPACT[] = "%Y%m%dT%H%M%SZ";

	inline Timestamp timestamp(char const *fmt)
	{
		char buf[MAX_TIMESTAMP_LEN + 1];

		Libc::with_libc([fmt, &buf] {
			::time_t t;
			::tm time;

			if (::time(&t) != -1 && ::gmtime_r(&t, &time)) {
				::strftime(buf, sizeof(buf), fmt, &time);
			} else {
				Genode::error("failed to produce timestamp");
				::strncpy(buf, "unknown_time", sizeof(buf));
			}
		});

		return Timestamp { buf };
	}
}
